# Generated by Django 2.0.4 on 2019-02-19 20:26

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('slackchat', '0008_auto_20190219_1745'),
    ]

    operations = [
        migrations.AddField(
            model_name='channel',
            name='published',
            field=models.BooleanField(default=False, help_text='Determines if the page should be live (for renderer)'),
        ),
        migrations.AlterField(
            model_name='channel',
            name='publish_time',
            field=models.DateTimeField(blank=True, help_text='Dateline.', null=True),
        ),
    ]
