# Generated by Django 2.0.4 on 2019-02-19 17:45

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('slackchat', '0007_message_serialized'),
    ]

    operations = [
        migrations.AlterField(
            model_name='channel',
            name='publish_time',
            field=models.DateTimeField(help_text='Dateline.', null=True),
        ),
    ]
